@@ -1,19 +1,18 @@
 use chrono::prelude::*;
-use markdown;
 use quick_xml::{events::Event, Reader, Writer};
 use serde::{Deserialize, Serialize};
 use std::{
     env, fs,
     io::{self, Cursor},
-    path::PathBuf,
+    path::{Path, PathBuf},
     str,
 };
 use structopt::StructOpt;
 
 #[derive(Clone, PartialEq, Serialize, Deserialize)]
 struct Entry {
+    id: String,
     name: String,
-    kebab: String,
     date: String,
     author: String,
     img: Option<String>,
@@ -23,6 +22,7 @@ struct Entry {
 #[derive(Serialize, Deserialize)]
 struct BlogFile {
     config: PathBuf,
+    config_dir: PathBuf,
     entries: Vec<Entry>,
 }
 
@@ -58,11 +58,15 @@ struct Args {
 fn main() -> Result<(), Box<dyn std::error::Error>> {
     setup()?;
     let blog_file: BlogFile = serde_json::from_str(&fs::read_to_string("blog/.config.json")?)?;
-    let config: Config = serde_json::from_str(&fs::read_to_string(&blog_file.config)?)?;
+    let config: Config = serde_json::from_str(&fs::read_to_string(
+        &blog_file.config_dir.join(&blog_file.config),
+    )?)?;
     let args = Args::from_args();
 
     if args.new {
         new_draft(blog_file)?
+    } else if blog_file.entries.is_empty() {
+        println!("No blog entries exist.")
     } else if args.publish {
         publish_draft(blog_file, config)?
     } else {
@@ -80,15 +84,15 @@ fn setup() -> Result<(), io::Error> {
         let x = x.expect("Error");
         x.file_type().unwrap().is_dir() && x.file_name().to_str().unwrap() == "blog"
     }) {
-        let mut config_path: PathBuf;
+        let mut config: String;
         println!("Blog Setup");
         loop {
             println!("Please input the path to your config file:");
             match read_input() {
                 Ok(input) => {
-                    config_path = PathBuf::from(input);
+                    config = input;
                     clear();
-                    match fs::read_to_string(&config_path) {
+                    match fs::read_to_string(&config) {
                         Ok(s) => match serde_json::from_str::<Config>(&s) {
                             Ok(_) => break,
                             _ => println!("Config file was not valid."),
@@ -99,12 +103,12 @@ fn setup() -> Result<(), io::Error> {
                 _ => println!("Error whilst reading input."),
             }
         }
-
+        let i: usize = config.rfind('/').unwrap_or(0) + 1;
         let f = BlogFile {
-            config: config_path,
+            config_dir: PathBuf::from(&config[..i]),
+            config: PathBuf::from(if i == 1 { "" } else { &config[i..] }),
             entries: Vec::new(),
         };
-
         fs::create_dir_all("blog/drafts/")?;
         fs::write("blog/.config.json", serde_json::to_string(&f)?)?;
         clear();
@@ -115,7 +119,6 @@ fn setup() -> Result<(), io::Error> {
 fn new_draft(mut blog_file: BlogFile) -> Result<(), io::Error> {
     println!("Please enter the title of the blog post:");
     let name = read_input()?;
-    let k = kebab(&name);
 
     println!("Please enter the name of the author:");
     let author = read_input()?;
@@ -123,8 +126,8 @@ fn new_draft(mut blog_file: BlogFile) -> Result<(), io::Error> {
     // Create draft
     fs::File::create(format!("blog/drafts/{}.md", name))?;
     blog_file.entries.push(Entry {
+        id: random_string::generate(14, "0123456789"),
         name,
-        kebab: k,
         author,
         published: false,
         date: String::new(),
@@ -135,28 +138,33 @@ fn new_draft(mut blog_file: BlogFile) -> Result<(), io::Error> {
 }
 
 fn delete(mut blog_file: BlogFile, config: Config) -> Result<(), Box<dyn std::error::Error>> {
-    println!("Please enter the number of the blog post you wish to delete:");
-    let choice = blog_file
-        .entries
-        .remove(display_choices(&blog_file.entries)?);
-
-    if choice.published {
-        fs::remove_file(format!("blog/{}.html", choice.kebab))?;
-
-        // Remove XML and HTML entries
-        remove_xml(config.rss, &choice)?;
-        remove_xml(config.blog, &choice)?;
+    if blog_file.entries.is_empty() {
+        println!("No blog entries to delete.")
     } else {
-        fs::remove_file(format!("blog/drafts/{}.md", &choice.name))?;
+        println!("Please enter the number of the blog post you wish to delete:");
+        let choice = blog_file
+            .entries
+            .remove(display_choices(&blog_file.entries)?);
+
+        if choice.published {
+            fs::remove_file(format!("blog/{}.html", choice.id))?;
+
+            // Remove XML and HTML entries
+            remove_xml(blog_file.config_dir.join(config.rss), &choice)?;
+            remove_xml(blog_file.config_dir.join(config.blog), &choice)?;
+        } else {
+            fs::remove_file(format!("blog/drafts/{}.md", &choice.name))?;
+        }
+
+        fs::write("blog/.config.json", serde_json::to_string(&blog_file)?)?;
     }
 
-    fs::write("blog/.config.json", serde_json::to_string(&blog_file)?)?;
     Ok(())
 }
 
-fn remove_xml(file: PathBuf, entry: &Entry) -> Result<(), Box<dyn std::error::Error>> {
-    let path = fs::read_to_string(&file)?;
-    let mut r = Reader::from_str(&path);
+fn remove_xml(path: PathBuf, entry: &Entry) -> Result<(), Box<dyn std::error::Error>> {
+    let file = fs::read_to_string(&path)?;
+    let mut r = Reader::from_str(&file);
     let mut w = Writer::new(Cursor::new(Vec::new()));
     let mut buf = Vec::<u8>::new();
     let mut found = false;
@@ -164,15 +172,26 @@ fn remove_xml(file: PathBuf, entry: &Entry) -> Result<(), Box<dyn std::error::Er
     // Loop over the xml tags
     loop {
         match r.read_event(&mut buf) {
+            Ok(Event::CData(e)) if !found => {
+                w.write(
+                    format!(
+                        "<![CDATA[{}]]>\n",
+                        str::from_utf8(&e.unescaped()?.into_owned())?
+                    )
+                    .as_bytes(),
+                )?;
+            }
             Ok(Event::Start(ref e))
                 if (e.name() == b"item" || e.name() == b"li")
                     && e.attributes().any(|a| {
-                        a.unwrap().value.into_owned() == xml_escape(&entry.kebab).into_bytes()
+                        a.unwrap().value.into_owned() == entry.id.clone().into_bytes()
                     }) =>
             {
                 found = true
             }
-            Ok(Event::End(ref e)) if e.name() == b"item" || e.name() == b"li" => found = false,
+            Ok(Event::End(ref e)) if found && (e.name() == b"item" || e.name() == b"li") => {
+                found = false
+            }
             Ok(Event::Eof) => break,
             Ok(e) if !found => w.write_event(e)?,
             Ok(_) => (),
@@ -180,7 +199,7 @@ fn remove_xml(file: PathBuf, entry: &Entry) -> Result<(), Box<dyn std::error::Er
         }
         buf.clear();
     }
-    fs::write(file, w.into_inner().into_inner())?;
+    fs::write(path, w.into_inner().into_inner())?;
     Ok(())
 }
 
@@ -188,7 +207,7 @@ fn publish_draft(
     mut blog_file: BlogFile,
     config: Config,
 ) -> Result<(), Box<dyn std::error::Error>> {
-    if blog_file.entries.len() == 0 {
+    if blog_file.entries.is_empty() {
         println!("No blog entries exist.")
     } else {
         // Read in choice
@@ -211,43 +230,60 @@ fn publish_draft(
         choices[i].date = Utc::now().to_rfc2822();
 
         // Create blog entry
-        insert_xml(&config.template, &config, &choices[i], &html, "template")?;
-
+        insert_xml(
+            &blog_file.config_dir.join(&config.template),
+            &config,
+            &choices[i],
+            &html,
+            "template",
+        )?;
         // Edit rolling blog and rss feed
-        insert_xml(&config.rss, &config, &choices[i], &html, "rss")?;
-        insert_xml(&config.blog, &config, &choices[i], &html, "blog")?;
+        insert_xml(
+            &blog_file.config_dir.join(&config.rss),
+            &config,
+            &choices[i],
+            &html,
+            "rss",
+        )?;
+        insert_xml(
+            &blog_file.config_dir.join(&config.blog),
+            &config,
+            &choices[i],
+            &html,
+            "blog",
+        )?;
 
         choices[i].published = true;
         choices.extend(list);
         blog_file.entries = choices.clone();
         fs::write("blog/.config.json", serde_json::to_string(&blog_file)?)?;
-        fs::remove_file(format!("blog/drafts/{}.md", kebab(&choices[i].name)))?;
+        fs::remove_file(format!("blog/drafts/{}.md", choices[i].name))?;
     }
     Ok(())
 }
 
 fn insert_xml(
-    file: &PathBuf,
+    path: &Path,
     config: &Config,
     entry: &Entry,
     html: &str,
     flag: &str,
 ) -> Result<(), Box<dyn std::error::Error>> {
-    let path = fs::read_to_string(&file)?;
-    let mut r = Reader::from_str(&path);
+    let file = fs::read_to_string(&path)?;
+    let mut r = Reader::from_str(&file);
     let mut w = Writer::new(Cursor::new(Vec::new()));
     let mut buf = Vec::<u8>::new();
 
     // Create template string
     let mut s = format!(
-        "<{size}>{name}</{size}><span>by {author}</span><time datetime='{rfc}'>{date}</time>",
+        "<{size}>{name}</{size}><span>by {author} </span><time datetime='{rfc}'>{date}</time>",
         name = &entry.name,
         author = &entry.author,
         rfc = &entry.date,
         size = if flag == "blog" { "h3" } else { "h1" },
         date = entry
             .date
-            .split(" ")
+            .split(' ')
             .take(4)
             .fold(String::new(), |acc, e| acc + " " + e)[1..]
             .to_string()
@@ -259,20 +295,19 @@ fn insert_xml(
     match flag {
         "rss" => {
             s = format!(
-                "<item id='{name}'>\n<title>{name}</title>\n<guid>{address}{kebab}</guid>\n<pubDate>{rfc}</pubDate>\n<description>\n<![CDATA[\n{s}\n{html}\n]]>\n</description>\n</item>",
+                "<item id='{id}'><title>{name}</title><guid>{address}{id}</guid><pubDate>{rfc}</pubDate><description><![CDATA[{s}{html}]]></description></item>",
+                id = entry.id,
                 name = xml_escape(&entry.name),
-                kebab = xml_escape(&entry.kebab),
                 address = config.blog_address,
                 rfc = &entry.date,
                 s = &s,
-                html = html,
+                html = html.replace('\n', ""),
             )
         }
         "blog" => {
             s = format!(
-                "<li id='{name}'><a href='{address}{kebab}'>{s}</a></li>",
-                name = xml_escape(&entry.name),
-                kebab = entry.kebab,
+                "<li id='{id}'><a href='{address}{id}'>{s}</a></li>",
+                id = entry.id,
                 address = config.blog_address,
                 s = s,
             )
@@ -288,9 +323,18 @@ fn insert_xml(
     // Loop over every tag
     loop {
         match r.read_event(&mut buf) {
+            Ok(Event::CData(e)) if found && count <= config.items => {
+                w.write(
+                    format!(
+                        "<![CDATA[{}]]>\n",
+                        str::from_utf8(&e.unescaped()?.into_owned())?
+                    )
+                    .as_bytes(),
+                )?;
+            }
             // Remove excess items on the rss feed
             Ok(Event::Start(e)) if flag == "rss" && e.name() == b"item" => {
-                count = count + 1;
+                count += 1;
                 found = true;
                 if count <= config.items {
                     w.write_event(Event::Start(e))?;
@@ -318,6 +362,8 @@ fn insert_xml(
                 }
             }
             Ok(Event::Eof) => break,
+
+            // Remove excess items on the rss feed
             Ok(_) if found && count > config.items => (),
             Ok(e) => w.write_event(e)?,
             Err(e) => panic!(
@@ -332,9 +378,9 @@ fn insert_xml(
 
     fs::write(
         if flag == "template" {
-            PathBuf::from(format!("blog/{}.html", entry.kebab))
+            PathBuf::from(format!("blog/{}.html", entry.id))
         } else {
-            file.clone()
+            path.to_path_buf()
         },
         w.into_inner().into_inner(),
     )?;
@@ -368,10 +414,6 @@ fn display_choices(blog_file: &[Entry]) -> Result<usize, Box<dyn std::error::Err
     Ok(input)
 }
 
-fn kebab(s: &str) -> String {
-    s.to_lowercase().replace(' ', "-")
-}
-
 fn xml_escape(s: &str) -> String {
     s.replace('\"', "&quot;")
         .replace('\'', "&apos;")
